@@ -11,29 +11,26 @@ log = tuned.logs.get()
 
 class DiskPlugin(hotplug.Plugin):
 	"""
-	`disk`::
-	
 	Plug-in for tuning various block device options. This plug-in can also
 	dynamically change the advanced power management and spindown timeout
 	setting for a drive according to the current drive utilization. The
 	dynamic tuning is controlled by the [option]`dynamic` and the global
 	[option]`dynamic_tuning` option in `tuned-main.conf`.
-	+
+
 	The disk plug-in operates on all supported block devices unless a
 	comma separated list of [option]`devices` is passed to it.
-	+
-	.Operate only on the sda block device
+
+	.Operate only on the `sda` block device
 	====
 	----
 	[disk]
-	# Comma separated list of devices, all devices if commented out.
 	devices=sda
 	----
 	====
-	+
+
 	The [option]`elevator` option sets the Linux I/O scheduler.
-	+
-	.Use the bfq I/O scheduler on xvda block device
+
+	.Use the bfq I/O scheduler on the `xvda` block device
 	====
 	----
 	[disk]
@@ -41,21 +38,21 @@ class DiskPlugin(hotplug.Plugin):
 	elevator=bfq
 	----
 	====
-	+
+
 	The [option]`scheduler_quantum` option only applies to the CFQ I/O
 	scheduler. It defines the number of I/O requests that CFQ sends to
 	one device at one time, essentially limiting queue depth. The default
 	value is 8 requests. The device being used may support greater queue
 	depth, but increasing the value of quantum will also increase latency,
 	especially for large sequential write work loads.
-	+
+
 	The [option]`apm` option sets the Advanced Power Management feature
 	on drives that support it. It corresponds to using the `-B` option of
 	the `hdparm` utility. The [option]`spindown` option puts the drive
 	into idle (low-power) mode, and also sets the standby (spindown)
 	timeout for the drive. It corresponds to using `-S` option of the
 	`hdparm` utility.
-	+
+
 	.Use a medium-agressive power management with spindown
 	====
 	----
@@ -64,7 +61,7 @@ class DiskPlugin(hotplug.Plugin):
 	spindown=6
 	----
 	====
-	+
+
 	The [option]`readahead` option controls how much extra data the
 	operating system reads from disk when performing sequential
 	I/O operations. Increasing the `readahead` value might improve
@@ -73,7 +70,7 @@ class DiskPlugin(hotplug.Plugin):
 	can be adjusted to sectors by specifying the suffix 's'. If the
 	suffix is specified, there must be at least one space between the
 	number and suffix (for example, `readahead=8192 s`).
-	+
+
 	.Set the `readahead` to 4MB unless already set to a higher value
 	====
 	----
@@ -137,7 +134,8 @@ class DiskPlugin(hotplug.Plugin):
 		return  device.device_type == "disk" and \
 			device.attributes.get("removable", None) == b"0" and \
 			(device.parent is None or \
-					device.parent.subsystem in ["scsi", "virtio", "xen", "nvme"])
+					device.parent.subsystem in ["scsi", "virtio", "xen", "nvme",
+												"mmc"])
 
 	def _hardware_events_init(self):
 		self._hardware_inventory.subscribe(self, "block", self._hardware_events_callback)
@@ -338,7 +336,7 @@ class DiskPlugin(hotplug.Plugin):
 		return self._sysfs_path(device, "queue/scheduler")
 
 	@command_set("elevator", per_device=True)
-	def _set_elevator(self, value, device, sim, remove):
+	def _set_elevator(self, value, device, instance, sim, remove):
 		sys_file = self._elevator_file(device)
 		if not sim:
 			self._cmd.write_to_file(sys_file, value, \
@@ -346,14 +344,14 @@ class DiskPlugin(hotplug.Plugin):
 		return value
 
 	@command_get("elevator")
-	def _get_elevator(self, device, ignore_missing=False):
+	def _get_elevator(self, device, instance, ignore_missing=False):
 		sys_file = self._elevator_file(device)
 		# example of scheduler file content:
 		# noop deadline [cfq]
 		return self._cmd.get_active_option(self._cmd.read_file(sys_file, no_error=ignore_missing))
 
 	@command_set("apm", per_device=True)
-	def _set_apm(self, value, device, sim, remove):
+	def _set_apm(self, value, device, instance, sim, remove):
 		if not self._is_hdparm_apm_supported(device):
 			if not sim:
 				log.info("apm option is not supported for device '%s'" % device)
@@ -369,7 +367,7 @@ class DiskPlugin(hotplug.Plugin):
 			return None
 
 	@command_get("apm")
-	def _get_apm(self, device, ignore_missing=False):
+	def _get_apm(self, device, instance, ignore_missing=False):
 		if not self._is_hdparm_apm_supported(device):
 			if not ignore_missing:
 				log.info("apm option is not supported for device '%s'" % device)
@@ -393,7 +391,7 @@ class DiskPlugin(hotplug.Plugin):
 		return value
 
 	@command_set("spindown", per_device=True)
-	def _set_spindown(self, value, device, sim, remove):
+	def _set_spindown(self, value, device, instance, sim, remove):
 		if not self._is_hdparm_apm_supported(device):
 			if not sim:
 				log.info("spindown option is not supported for device '%s'" % device)
@@ -409,7 +407,7 @@ class DiskPlugin(hotplug.Plugin):
 			return None
 
 	@command_get("spindown")
-	def _get_spindown(self, device, ignore_missing=False):
+	def _get_spindown(self, device, instance, ignore_missing=False):
 		if not self._is_hdparm_apm_supported(device):
 			if not ignore_missing:
 				log.info("spindown option is not supported for device '%s'" % device)
@@ -432,7 +430,7 @@ class DiskPlugin(hotplug.Plugin):
 		return v
 
 	@command_set("readahead", per_device=True)
-	def _set_readahead(self, value, device, sim, remove):
+	def _set_readahead(self, value, device, instance, sim, remove):
 		sys_file = self._readahead_file(device)
 		val = self._parse_ra(value)
 		if val is None:
@@ -444,7 +442,7 @@ class DiskPlugin(hotplug.Plugin):
 		return val
 
 	@command_get("readahead")
-	def _get_readahead(self, device, ignore_missing=False):
+	def _get_readahead(self, device, instance, ignore_missing=False):
 		sys_file = self._readahead_file(device)
 		value = self._cmd.read_file(sys_file, no_error=ignore_missing).strip()
 		if len(value) == 0:
@@ -452,31 +450,31 @@ class DiskPlugin(hotplug.Plugin):
 		return int(value)
 
 	@command_custom("readahead_multiply", per_device=True)
-	def _multiply_readahead(self, enabling, multiplier, device, verify, ignore_missing):
+	def _multiply_readahead(self, enabling, multiplier, device, verify, ignore_missing, instance):
 		if verify:
 			return None
 		storage_key = self._storage_key(
 				command_name = "readahead_multiply",
 				device_name = device)
 		if enabling:
-			old_readahead = self._get_readahead(device)
+			old_readahead = self._get_readahead(device, instance)
 			if old_readahead is None:
 				return
 			new_readahead = int(float(multiplier) * old_readahead)
 			self._storage.set(storage_key, old_readahead)
-			self._set_readahead(new_readahead, device, False)
+			self._set_readahead(new_readahead, device, instance, False, False)
 		else:
 			old_readahead = self._storage.get(storage_key)
 			if old_readahead is None:
 				return
-			self._set_readahead(old_readahead, device, False)
+			self._set_readahead(old_readahead, device, instance, False, False)
 			self._storage.unset(storage_key)
 
 	def _scheduler_quantum_file(self, device):
 		return self._sysfs_path(device, "queue/iosched/quantum")
 
 	@command_set("scheduler_quantum", per_device=True)
-	def _set_scheduler_quantum(self, value, device, sim, remove):
+	def _set_scheduler_quantum(self, value, device, instance, sim, remove):
 		sys_file = self._scheduler_quantum_file(device)
 		if not sim:
 			self._cmd.write_to_file(sys_file, "%d" % int(value), \
@@ -484,7 +482,7 @@ class DiskPlugin(hotplug.Plugin):
 		return value
 
 	@command_get("scheduler_quantum")
-	def _get_scheduler_quantum(self, device, ignore_missing=False):
+	def _get_scheduler_quantum(self, device, instance, ignore_missing=False):
 		sys_file = self._scheduler_quantum_file(device)
 		value = self._cmd.read_file(sys_file, no_error=ignore_missing).strip()
 		if len(value) == 0:
